#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import tempfile
import logging

import cli

MENDER_ARTIFACT_TOOL='tests/mender-artifact'

def create_artifact_file(outpath):
    with tempfile.NamedTemporaryFile(prefix='menderin') as infile:
        logging.info('writing mender artifact to %s', outpath)

        infile.write(b'bogus test data')
        infile.flush()

        c = cli.Cli(MENDER_ARTIFACT_TOOL)
        r = c.run('write', 'rootfs-image',
                  '--device-type', 'device-foo',\
                  '--file', infile.name, \
                  '--artifact-name', 'artifact-foo',\
                  '--output-path', outpath)

        if r.returncode != 0:
            msg = 'mender-artifact failed with code {}, \nstdout: \n{}stderr: {}\n'.format(r.returncode, r.stdout, r.stderr)
            logging.error(msg)
            raise RuntimeError(msg)

        logging.info('mender artifact written to %s', outpath)



#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
from pathlib import Path
import os

import pytest

import docker

USER_HOME = str(Path.home())
DEFAULT_TOKEN_PATH = os.path.join(USER_HOME,'.cache', 'mender', 'authtoken')

@pytest.yield_fixture(scope="class")
def single_user():
    r = docker.exec('mender-useradm', \
                    docker.BASE_COMPOSE_FILES, \
                    '/usr/bin/useradm', \
                    'create-user', \
                    '--username' , 'user@tenant.com',\
                    '--password' , 'youcantguess')

    assert r.returncode == 0, r.stderr
    yield
    clean_useradm_db()


def clean_useradm_db():
    r = docker.exec('mender-mongo', \
                    docker.BASE_COMPOSE_FILES, \
                    'mongo', 'useradm', '--eval', 'db.dropDatabase()')

    assert r.returncode == 0, r.stderr

def expect_output(stream, *expected):
        for e in expected:
            assert e in stream, 'expected string {} not found in stream'.format(e)



#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import os

import requests


class Deployments:
    def __init__(self, token, url='https://mender-api-gateway/api/management/v1/deployments'):
        self.url=url
        self.token=token

    def get_artifacts(self):
        auth = {'Authorization': 'Bearer {}'.format(self.token)}
        return requests.get(self.make_api_url('/artifacts'), verify=False, headers=auth)

    def make_api_url(self, path):
        return os.path.join(self.url,
                            path if not path.startswith("/") else path[1:])



#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
from minio import Minio
from minio.error import ResponseError

def cleanup_mender_storage(bucket='mender-artifact-storage'):
    client = Minio('minio.s3.docker.mender.io:9000',
                    access_key='minio',
                    secret_key='minio123',
                    region='us-east-1',
                    secure=False)

    objs = client.list_objects(bucket)
    client.remove_objects(bucket, objs)



#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import os
from pathlib import Path

import pytest

import cli
import artifact
from common import single_user, expect_output, DEFAULT_TOKEN_PATH
import api
import docker
import s3


@pytest.yield_fixture(scope="function")
def logged_in_single_user(single_user):
    c = cli.Cli()
    r = c.run('login', \
              '--server', 'https://mender-api-gateway', \
              '--skip-verify', \
              '--username', 'user@tenant.com', \
              '--password', 'youcantguess')

    assert r.returncode == 0, r.stderr
    yield
    os.remove(DEFAULT_TOKEN_PATH)


@pytest.yield_fixture(scope="function")
def valid_artifact():
    path = '/tests/foo-artifact'
    artifact.create_artifact_file(path)
    yield path
    os.remove(path)

@pytest.yield_fixture(scope="function")
def clean_deployments_db():
    yield
    r = docker.exec('mender-mongo-deployments', \
                    docker.BASE_COMPOSE_FILES, \
                    'mongo', 'deployment_service', '--eval', 'db.dropDatabase()')

@pytest.yield_fixture(scope="function")
def clean_mender_storage():
    yield
    s3.cleanup_mender_storage()

class TestArtifactUpload:
    @pytest.mark.usefixtures('clean_deployments_db', 'clean_mender_storage')
    def test_ok(self, logged_in_single_user, valid_artifact):
        c = cli.Cli()
        r = c.run('--server', 'https://mender-api-gateway', \
                  '--skip-verify', \
                  'artifacts', 'upload', \
                  '--description', 'foo',
                  valid_artifact)

        assert r.returncode==0, r.stderr
        expect_output(r.stdout, 'upload successful')

        token = Path(DEFAULT_TOKEN_PATH).read_text()

        dapi = api.Deployments(token)
        r = dapi.get_artifacts()

        assert r.status_code == 200

        artifacts = r.json()
        assert len(artifacts) == 1

        artifact = artifacts[0]

        assert artifact['name'] == 'artifact-foo'
        assert artifact['description'] == 'foo'
        assert artifact['device_types_compatible'] == ['device-foo']

    def test_error_no_login(self, valid_artifact):
        c = cli.Cli()
        r = c.run('--server', 'https://mender-api-gateway', \
                  '--skip-verify', \
                  'artifacts', 'upload', \
                  '--description', 'foo',
                  valid_artifact)

        assert r.returncode!=0
        expect_output(r.stderr, 'FAILURE', 'Please Login first')

    def test_error_no_server(self, valid_artifact, clean_deployments_db):
        c = cli.Cli()
        r = c.run('--skip-verify', \
                  'artifacts', 'upload', \
                  '--description', 'foo',
                  valid_artifact)

        assert r.returncode!=0
        expect_output(r.stderr, '"server" not set')



#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import cli

BASE_COMPOSE_FILES = [
    "/tests/integration/docker-compose.yml",
    "/tests/integration/docker-compose.storage.minio.yml",
    "/tests/integration/docker-compose.testing.yml",
]

def exec(service, files, *argv):
    c = cli.Cli('/usr/bin/docker-compose')

    # set by run-test-environment
    args = ['-p', 'acceptance-tests']

    for f in files:
        args += ['-f', f]

    args += ['exec', '-T', service] + list(argv)

    return c.run(*args)



#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import os

import pytest

from common import single_user, expect_output, DEFAULT_TOKEN_PATH
import cli

@pytest.yield_fixture(scope="function")
def cleanup_token(request):
    yield
    os.remove(request.param)

class TestLogin:
    @pytest.mark.parametrize('cleanup_token', [DEFAULT_TOKEN_PATH], indirect=True)
    def test_ok(self, single_user, cleanup_token):
        c = cli.Cli()

        r = c.run('login', \
                  '--server', 'https://mender-api-gateway', \
                  '--skip-verify', \
                  '--username', 'user@tenant.com', \
                  '--password', 'youcantguess')

        assert r.returncode == 0, r.stderr

        self.__check_token_at(DEFAULT_TOKEN_PATH)
        expect_output(r.stdout, \
                             'login successful')

    @pytest.mark.parametrize('cleanup_token', ['/tests/authtoken'], indirect=True)
    def test_ok_custom_path(self, single_user, cleanup_token):
        c = cli.Cli()

        custom_path = '/tests/authtoken'

        r = c.run('login', \
                  '--server', 'https://mender-api-gateway', \
                  '--skip-verify', \
                  '--token', '/tests/authtoken', \
                  '--username', 'user@tenant.com', \
                  '--password', 'youcantguess')

        assert r.returncode == 0, r.stderr

        self.__check_token_at(custom_path)
        expect_output(r.stdout, \
                             'login successful')

    @pytest.mark.parametrize('cleanup_token', [DEFAULT_TOKEN_PATH], indirect=True)
    def test_ok_verbose(self, single_user, cleanup_token):
        c = cli.Cli()

        r = c.run('login', \
                  '--server', 'https://mender-api-gateway', \
                  '--skip-verify', \
                  '--verbose', \
                  '--username', 'user@tenant.com', \
                  '--password', 'youcantguess')

        assert r.returncode == 0, r.stderr

        self.__check_token_at(DEFAULT_TOKEN_PATH)
        expect_output(r.stdout, \
                             'creating directory',
                             'saved token to',
                             'login successful')

    def test_error_wrong_creds(self, single_user):
        c = cli.Cli()

        r = c.run('login', \
                  '--server', 'https://mender-api-gateway', \
                  '--skip-verify', \
                  '--username', 'notfound@tenant.com', \
                  '--password', 'youcantguess')

        assert r.returncode != 0

        expect_output(r.stderr, 'FAILURE: login failed with status 401')

    def test_error_no_server(self, single_user):
        c = cli.Cli()

        r = c.run('login', \
                  '--skip-verify', \
                  '--username', 'user@tenant.com', \
                  '--password', 'youcantguess')

        assert r.returncode != 0

        expect_output(r.stderr, '"server" not set')

    def __check_token_at(self, path):
        assert os.path.isfile(path)



#!/usr/bin/python
# Copyright 2018 Mender Software AS
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import subprocess

MENDER_CLI='tests/mender-cli'

class Cli:
    """Simple wrapper for subprocess"""
    def __init__(self, path=MENDER_CLI):
        self.path = path

    def run(self, *argv):
        """Returns a CompletedProcess wrapped in CliResult"""
        args = [self.path] + list(argv)
        completed = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return CliResult(completed)

class CliResult:
    """Wraps CompletedProcess, decodes output to strings"""
    def __init__(self, completed_process):
        self.completed_process = completed_process

        self.returncode = completed_process.returncode
        self.stdout = self.completed_process.stdout.decode('utf-8')
        self.stderr = self.completed_process.stderr.decode('utf-8')
